@@ -1,4 +1,4 @@
-/* This file is part of the KDE libraries
+/* This file is part of the dbusmenu-qt library
    Copyright 2010 Canonical
    Author: Aurelien Gateau <aurelien.gateau@canonical.com>
 
@@ -21,6 +21,10 @@
 #ifndef TESTUTILS_H
 #define TESTUTILS_H
 
+// Local
+#include <debug_p.h>
+#include <dbusmenutypes_p.h>
+
 // Qt
 #include <QObject>
 #include <QMenu>
@@ -44,6 +48,23 @@ public Q_SLOTS:
     {
         append(QVariantList() << v1 << v2);
     }
+
+    void receiveCall(DBusMenuItemList itemList, DBusMenuItemKeysList removedPropsList)
+    {
+        QVariantList propsIds;
+        Q_FOREACH(DBusMenuItem item, itemList) {
+            propsIds << item.id;
+        }
+        QVariantList removedPropsIds;
+        Q_FOREACH(DBusMenuItemKeys props, removedPropsList) {
+            removedPropsIds << props.id;
+        }
+
+        QVariantList args;
+        args.push_back(propsIds);
+        args.push_back(removedPropsIds);
+        append(args);
+    }
 };
 
 class MenuFiller : public QObject
